"""
Constants
"""

# Standard Library
import os

PACKAGE_NAME = "timezones"
APP_BASE_DIR = os.path.join(os.path.dirname(__file__))
APP_STATIC_DIR = os.path.join(APP_BASE_DIR, "static", PACKAGE_NAME)

AA_TIMEZONE_DEFAULT_PANELS: list[dict[str, str | dict[str, str]]] = [
    {
        "panel_name": "US / Pacific",
        "timezone": {
            "timezone_name": "US/Pacific",
            "panel_id": "us-pacific",
        },
    },
    {
        "panel_name": "US / Mountain",
        "timezone": {
            "timezone_name": "US/Mountain",
            "panel_id": "us-mountain",
        },
    },
    {
        "panel_name": "US / Central",
        "timezone": {
            "timezone_name": "US/Central",
            "panel_id": "us-central",
        },
    },
    {
        "panel_name": "US / Eastern",
        "timezone": {
            "timezone_name": "US/Eastern",
            "panel_id": "us-eastern",
        },
    },
    {
        "panel_name": "EU / Western",
        "timezone": {
            "timezone_name": "Europe/London",
            "panel_id": "eu-western",
        },
    },
    {
        "panel_name": "EU / Central",
        "timezone": {
            "timezone_name": "Europe/Berlin",
            "panel_id": "eu-central",
        },
    },
    {
        "panel_name": "EU / Eastern",
        "timezone": {
            "timezone_name": "Europe/Istanbul",
            "panel_id": "eu-eastern",
        },
    },
    {
        "panel_name": "Russia / Moscow",
        "timezone": {
            "timezone_name": "Europe/Moscow",
            "panel_id": "russia-moscow",
        },
    },
    {
        "panel_name": "China / Shanghai",
        "timezone": {
            "timezone_name": "Asia/Shanghai",
            "panel_id": "asia-shanghai",
        },
    },
    {
        "panel_name": "Australia / Sydney",
        "timezone": {
            "timezone_name": "Australia/ACT",
            "panel_id": "australia-act",
        },
    },
]



# Generated by Django 4.0.10 on 2023-09-22 23:00

# Django
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("timezones", "0004_delete_aatimezones"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="timezonedata",
            options={
                "default_permissions": (),
                "verbose_name": "Timezone data",
                "verbose_name_plural": "Timezone data",
            },
        ),
    ]



"""
Test cases for the models of the timezones app.
"""

# Django
from django.test import TestCase

# AA Time Zones
from timezones.models import TimezoneData, Timezones


class TimezoneDataModelTests(TestCase):
    """
    Test cases for the TimezoneData model
    """

    def test_timezone_data_creation(self):
        """
        Test creating a timezone data object

        :return:
        :rtype:
        """

        timezone_data = TimezoneData.objects.create(
            timezone_name="UTC", utc_offset="+00:00", panel_id="utc-panel"
        )

        self.assertIsInstance(timezone_data, TimezoneData)
        self.assertEqual(timezone_data.__str__(), "UTC")


class TimezonesModelTests(TestCase):
    """
    Test cases for the Timezones model
    """

    def setUp(self):
        """
        Set up test cases

        :return:
        :rtype:
        """

        self.timezone_data = TimezoneData.objects.create(
            timezone_name="UTC", utc_offset="+00:00", panel_id="utc-panel"
        )

    def test_timezones_creation_with_enabled_status(self):
        """
        Test creating a timezone object with enabled status

        :return:
        :rtype:
        """

        timezone = Timezones.objects.create(
            panel_name="UTC Panel", timezone=self.timezone_data, is_enabled=True
        )

        self.assertIsInstance(timezone, Timezones)
        self.assertEqual(timezone.__str__(), "UTC Panel")
        self.assertTrue(timezone.is_enabled)

    def test_timezones_creation_with_disabled_status(self):
        """
        Test creating a timezone object with disabled status

        :return:
        :rtype:
        """

        timezone = Timezones.objects.create(
            panel_name="UTC Panel", timezone=self.timezone_data, is_enabled=False
        )

        self.assertIsInstance(timezone, Timezones)
        self.assertEqual(timezone.__str__(), "UTC Panel")
        self.assertFalse(timezone.is_enabled)



"""
Tests for the admin module.
"""

# Standard Library
from unittest.mock import Mock

# Django
from django.contrib import admin
from django.contrib.auth.models import User
from django.test import TestCase

# AA Time Zones
from timezones.admin import TimezonesAdmin
from timezones.models import TimezoneData, Timezones


class TestTimezonesAdmin(TestCase):
    """
    TimezonesAdmin tests
    """

    def setUp(self):
        """
        Setup

        :return:
        :rtype:
        """

        self.user = User.objects.create_user(username="test", password="test")
        timezone_data = TimezoneData.objects.create(
            timezone_name="UTC", utc_offset="+00:00", panel_id="utc"
        )
        self.timezone = Timezones.objects.create(
            panel_name="test", timezone=timezone_data, is_enabled=True
        )
        self.admin = TimezonesAdmin(Timezones, admin.site)

    def test_should_display_correct_fields(self):
        """
        Test should display correct fields

        :return:
        :rtype:
        """

        self.assertEqual(
            self.admin.list_display, ("_panel_name", "_timezone", "is_enabled")
        )
        self.assertEqual(self.admin.ordering, ("panel_name",))
        self.assertEqual(self.admin.list_filter, ("is_enabled",))

    def test_should_return_correct_panel_name(self):
        """
        Test should return correct panel name

        :return:
        :rtype:
        """

        self.assertEqual(self.admin._panel_name(self.timezone), "test")

    def test_should_return_correct_timezone(self):
        """
        Test should return correct timezone

        :return:
        :rtype:
        """

        self.assertEqual(self.admin._timezone(self.timezone), "UTC")

    def test_should_mark_as_active(self):
        """
        Test should mark as active

        :return:
        :rtype:
        """

        self.timezone.is_enabled = False
        self.timezone.save()
        request = Mock()
        self.admin.mark_as_active(
            request, Timezones.objects.filter(id=self.timezone.id)
        )
        self.timezone.refresh_from_db()
        self.assertTrue(self.timezone.is_enabled)

    def test_should_mark_as_inactive(self):
        """
        Test should mark as inactive

        :return:
        :rtype:
        """

        self.timezone.is_enabled = True
        self.timezone.save()
        request = Mock()
        self.admin.mark_as_inactive(
            request, Timezones.objects.filter(id=self.timezone.id)
        )
        self.timezone.refresh_from_db()
        self.assertFalse(self.timezone.is_enabled)



"""
The views
"""

# Django
from django.shortcuts import render

# AA Time Zones
from timezones import __title__
from timezones.constants import AA_TIMEZONE_DEFAULT_PANELS
from timezones.models import Timezones


def index(request, timestamp: str = None):
    """
    Index view

    :param request:
    :param timestamp:
    :return:
    """

    timezones = (
        Timezones.objects.select_related("timezone")
        .filter(is_enabled=True)
        .order_by("panel_name")
    )

    if not timezones:
        timezones = AA_TIMEZONE_DEFAULT_PANELS

    context = {"title": __title__, "timezones": timezones, "timestamp": timestamp}

    return render(
        request=request, template_name="timezones/index.html", context=context
    )



"""
loading timezone data into DB
"""

# Standard Library
import datetime

# Third Party
import pytz

# Django
from django.core.management.base import BaseCommand

# AA Time Zones
from timezones.models import TimezoneData


def get_input(text):
    """
    Wrapped input to enable tz import
    """

    return input(text)


class Command(BaseCommand):
    """
    Import timezones
    """

    help = "Imports timezone data"

    def _import_timezone_data(self) -> None:
        """
        Import time zone data
        :return:
        """

        timezones_imported = 0
        timezones_updated = 0

        for timezone_name in pytz.common_timezones:
            if timezone_name == "UTC":
                break

            timezone_panel_id = timezone_name.replace("/", "-").lower()
            timezone_utc_offset = datetime.datetime.now(
                pytz.timezone(timezone_name)
            ).strftime("%z")

            timezone, created = TimezoneData.objects.update_or_create(
                timezone_name=timezone_name,
                panel_id=timezone_panel_id,
                defaults={"utc_offset": timezone_utc_offset},
            )

            if created:
                action = "Importing"
                timezones_imported += 1
            else:
                action = "Updating"
                timezones_updated += 1

            self.stdout.write(
                f"{action} timezone '{timezone.timezone_name}' "
                f"with UTC offset of '{timezone.utc_offset}' "
                f"and panel ID of '{timezone.panel_id}'"
            )

        self.stdout.write(
            f"Import/Update done with {timezones_imported} new timezones imported "
            f"and {timezones_updated} timezones updated because they were already "
            "in the DB."
        )

    def handle(self, *args, **options):  # pylint: disable=unused-argument
        """
        Ask before running ...
        :param args:
        :param options:
        """

        self.stdout.write(
            "Timezones will be imported. "
            "Previously imported timezones will not be replaced or overwritten."
        )

        user_input = get_input("Are you sure you want to proceed? (yes/no)?")

        if user_input == "yes":
            self.stdout.write("Starting import of time zones. Please stand by.")
            self._import_timezone_data()
            self.stdout.write(self.style.SUCCESS("Timezones Import complete!"))
        else:
            self.stdout.write(self.style.WARNING("Aborted."))
